package com.sun.jmx.remote.security;

import java.util.HashMap;
import java.util.Collections;
import javax.security.auth.login.AppConfigurationEntry;
import javax.security.auth.login.Configuration;
import java.io.IOException;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import com.sun.jmx.remote.util.EnvHelp;
import java.security.PrivilegedAction;
import javax.security.auth.Subject;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import javax.security.auth.login.LoginException;
import java.security.PrivilegedExceptionAction;
import javax.security.auth.AuthPermission;
import java.util.Map;
import com.sun.jmx.remote.util.ClassLogger;
import javax.security.auth.login.LoginContext;
import javax.management.remote.JMXAuthenticator;

public final class JMXPluggableAuthenticator implements JMXAuthenticator
{
    private LoginContext loginContext;
    private String username;
    private String password;
    private static final String LOGIN_CONFIG_PROP = "jmx.remote.x.login.config";
    private static final String LOGIN_CONFIG_NAME = "JMXPluggableAuthenticator";
    private static final String PASSWORD_FILE_PROP = "jmx.remote.x.password.file";
    private static final ClassLogger logger;
    
    public JMXPluggableAuthenticator(Map<?, ?> map) {
        String name = null;
        final String s = null;
        if (map != null) {
            name = (String)map.get("jmx.remote.x.login.config");
            s = (String)map.get("jmx.remote.x.password.file");
        }
        try {
            if (name != null) {
                this.loginContext = new LoginContext(name, new JMXPluggableAuthenticator.JMXCallbackHandler());
            }
            else {
                SecurityManager securityManager = System.getSecurityManager();
                if (securityManager != null) {
                    securityManager.checkPermission(new AuthPermission("createLoginContext.JMXPluggableAuthenticator"));
                }
                try {
                    this.loginContext = (LoginContext)AccessController.doPrivileged(new PrivilegedExceptionAction<LoginContext>() {
                        @Override
                        public LoginContext run() throws LoginException {
                            return new LoginContext("JMXPluggableAuthenticator", null, new JMXPluggableAuthenticator.JMXCallbackHandler(), new FileLoginConfig(s));
                        }
                    });
                }
                catch (PrivilegedActionException ex) {
                    throw (LoginException)ex.getException();
                }
            }
        }
        catch (LoginException ex2) {
            authenticationFailure("authenticate", ex2);
        }
        catch (SecurityException ex3) {
            authenticationFailure("authenticate", ex3);
        }
    }
    
    @Override
    public Subject authenticate(Object o) {
        if (!(o instanceof String[])) {
            if (o == null) {
                authenticationFailure("authenticate", "Credentials required");
            }
            authenticationFailure("authenticate", "Credentials should be String[] instead of " + o.getClass().getName());
        }
        String[] array = (String[])(String[])o;
        if (array.length != 2) {
            authenticationFailure("authenticate", "Credentials should have 2 elements not " + array.length);
        }
        this.username = array[0];
        this.password = array[1];
        if (this.username == null || this.password == null) {
            authenticationFailure("authenticate", "Username or password is null");
        }
        try {
            this.loginContext.login();
            final Subject subject = this.loginContext.getSubject();
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    subject.setReadOnly();
                    return null;
                }
            });
            return subject;
        }
        catch (LoginException ex) {
            authenticationFailure("authenticate", ex);
            return null;
        }
    }
    
    private static void authenticationFailure(String s, String str) throws SecurityException {
        String string = "Authentication failed! " + str;
        SecurityException ex = new SecurityException(string);
        logException(s, string, ex);
        throw ex;
    }
    
    private static void authenticationFailure(String s, Exception cause) throws SecurityException {
        String s2;
        SecurityException ex;
        if ((cause instanceof SecurityException)) {
            s2 = cause.getMessage();
            ex = (SecurityException)cause;
        }
        else {
            s2 = "Authentication failed! " + cause.getMessage();
            SecurityException throwable = new SecurityException(s2);
            EnvHelp.initCause(throwable, cause);
            ex = throwable;
        }
        logException(s, s2, ex);
        throw ex;
    }
    
    private static void logException(String s, String msg, Exception t) {
        if (JMXPluggableAuthenticator.logger.traceOn()) {
            JMXPluggableAuthenticator.logger.trace(s, msg);
        }
        if (JMXPluggableAuthenticator.logger.debugOn()) {
            JMXPluggableAuthenticator.logger.debug(s, t);
        }
    }
    
    static {
        logger = new ClassLogger("javax.management.remote.misc", "JMXPluggableAuthenticator");
    }
    
    private final class JMXCallbackHandler implements CallbackHandler
    {
        @Override
        public void handle(Callback[] array) throws IOException, UnsupportedCallbackException {
            for (int i = 0; i < array.length; ++i) {
                if ((array[i] instanceof NameCallback)) {
                    ((NameCallback)array[i]).setName(JMXPluggableAuthenticator.this.username);
                }
                else if ((array[i] instanceof PasswordCallback)) {
                    ((PasswordCallback)array[i]).setPassword(JMXPluggableAuthenticator.this.password.toCharArray());
                }
                else {
                    throw new UnsupportedCallbackException(array[i], "Unrecognized Callback");
                }
            }
        }
    }
    
    private static class FileLoginConfig extends Configuration
    {
        private AppConfigurationEntry[] entries;
        private static final String FILE_LOGIN_MODULE;
        private static final String PASSWORD_FILE_OPTION = "passwordFile";
        
        public FileLoginConfig(String s) {
            HashMap emptyMap;
            if (s != null) {
                emptyMap = new HashMap(1);
                emptyMap.put("passwordFile", s);
            }
            else {
                emptyMap = Collections.emptyMap();
            }
            this.entries = new AppConfigurationEntry[] { new AppConfigurationEntry(JMXPluggableAuthenticator.FileLoginConfig.FILE_LOGIN_MODULE, AppConfigurationEntry.LoginModuleControlFlag.REQUIRED, emptyMap) };
        }
        
        @Override
        public AppConfigurationEntry[] getAppConfigurationEntry(String s) {
            return s.equals("JMXPluggableAuthenticator") ? this.entries : null;
        }
        
        @Override
        public void refresh() {
        }
        
        static {
            FILE_LOGIN_MODULE = FileLoginModule.class.getName();
        }
    }
}
